import Template from './Template.js';
import DateUtil from '@midefos/utils/src/DateUtil.js'

export default class TableColumns {

  // TEXT
  static text(key, text) {
    const textConfig = { title: text, field: key };
    return { ...TableColumns._defaultTextConfig(), ...textConfig };
  }

  static type(key, text) {
    const typeConfig = {
      headerFilter: 'list', headerFilterParams: {
        valuesLookup: true,
      }
    };
    return { ...TableColumns.text(key, text), ...typeConfig };
  }

  // LINKS
  static link(textKey, URLKey, text) {
    const linkConfig = {
      title: text, field: textKey,
      formatter: "link", formatterParams: { urlField: URLKey, target: "_" }
    }
    return { ...TableColumns._defaultTextConfig(), ...linkConfig };
  }

  static internaLink(textKey, URL, text) {
    const internalLinkConfig = {
      title: text, field: textKey,
      formatter: "link", formatterParams: { url: URL }
    }
    return { ...TableColumns._defaultTextConfig(), ...internalLinkConfig };
  }

  // IMAGE
  static image(field, width = 75, height = 75) {
    const imageConfig = {
      field,
      formatter: "image", formatterParams: {
        height: `${height}px`,
        width: `${width}px`,
      },
      tooltip: false
    }
    return { ...TableColumns._defaultConfig(), ...imageConfig };
  }

  // NUMERIC 
  static number(field, title) {
    return this.decimal(field, title, 0);
  }

  static decimal(field, title, decimals = 2) {
    const config = {
      field, title,
      hozAlign: 'right',
      formatter: (cell) => {
        let value = cell.getValue();
        if (!value) return 0;
        if (typeof value === 'string') {
          value = Number(value);
        }
        return value.toFixed(decimals);
      }
    }
    return { ...TableColumns._defaultTextConfig(), ...config };
  }

  // PRICE
  static price(key, text = 'Price') {
    const priceConfig = {
      title: text, field: key,
      formatter: (cell) => {
        const item = cell.getRow().getData();
        return TableColumns._buildPrice(item, key);
      }
    }
    return { ...TableColumns._defaultTextConfig(), ...priceConfig };
  }

  static _buildPrice(item, key) {
    let text = `<div>${item[key]}€</div>`;
    return text;
  }

  // PERFORMANCE
  static nanos(key, text) {
    const config = TableColumns._defaultTextConfig();
    const performanceConfig = {
      field: key, title: text,
      formatter: (cell) => {
        const nanos = cell.getValue();
        if (!nanos) return null;

        if (nanos < 1000) {
          return nanos.toFixed(2) + ' ns';
        }

        const microseconds = nanos / 1000;
        if (microseconds < 1000) {
          return microseconds.toFixed(2) + ' µs';
        }

        const milliseconds = microseconds / 1000;
        if (milliseconds < 1000) {
          return milliseconds.toFixed(2) + ' ms';
        }

        const seconds = milliseconds / 1000;
        if (seconds < 60) {
          return seconds.toFixed(2) + ' secs';
        }

        const minutes = seconds / 60;
        if (minutes < 60) {
          return minutes.toFixed(2) + ' mins';
        }

        return (minutes / 60).toFixed(2) + ' hrs';
      }
    }
    return { ...config, ...performanceConfig };
  }

  // SIZE 
  static bytes(key, text) {
    const sizeConfig = {
      field: key, title: text,
      formatter: (cell) => {
        const bytes = cell.getValue();
        if (!bytes) return null;

        if (bytes < 1024) {
          return bytes.toFixed(2) + ' B';
        }

        const kilobytes = bytes / 1024;
        if (kilobytes < 1024) {
          return kilobytes.toFixed(2) + ' kB';
        }

        const megabytes = kilobytes / 1024;
        if (megabytes < 1024) {
          return megabytes.toFixed(2) + ' MB';
        }

        const gigabytes = megabytes / 1024;
        if (gigabytes < 1024) {
          return gigabytes.toFixed(2) + ' GB';
        }

        return (gigabytes / 1024).toFixed(2) + ' TB';
      }
    }
    return { ...TableColumns._defaultTextConfig(), ...sizeConfig };
  }

  // DATE
  static unix_timestamp(field, title) {
    const datetimeConfig = {
      field, title,
      formatter: (cell) => {
        const value = cell.getValue();
        if (!value) return null;
        return new Date(value * 1000).toLocaleString();
      }
    }
    return { ...TableColumns._defaultTextConfig(), ...datetimeConfig };
  }

  static datetime(field, title) {
    const config = {
      field, title,
      formatter: (cell) => {
        const value = cell.getValue();
        if (!value) return null;
        return new Date(value).toLocaleString();
      }
    }
    return { ...TableColumns._defaultTextConfig(), ...config };
  }

  static date(field, title) {
    const dateConfig = {
      field, title,
      formatter: (cell) => {
        const value = cell.getValue();
        if (!value) return null;
        return new Date(value).toLocaleDateString();
      }
    }
    return { ...TableColumns._defaultTextConfig(), ...dateConfig };
  }

  static time(field, title) {
    const timeConfig = {
      field, title,
      formatter: (cell) => {
        const value = cell.getValue();
        if (!value) return null;
        return new Date(value).toLocaleTimeString();
      }
    }
    return { ...TableColumns._defaultTextConfig(), ...timeConfig };
  }

  static dateDiff(startKey, endKey, title) {
    const dateDiffConfig = {
      title,
      formatter: (cell) => {
        const item = cell.getRow().getData();
        return DateUtil.millisecondsToDHMS(DateUtil.difference(new Date(item[endKey]), new Date(item[startKey])));
      }
    }
    return { ...TableColumns._defaultTextConfig(), ...dateDiffConfig };
  }

  static unixElapsed(field, title) {
    const unixElapsedConfig = {
      field, title,
      formatter: (cell) => {
        const value = cell.getValue();
        if (!value) return null;

        const elapsed = Date.now() - (value * 1_000);
        return DateUtil.millisecondsToMS(elapsed);
      }
    }
    return { ...TableColumns._defaultTextConfig(), ...unixElapsedConfig };
  }

  // QUANTITY
  static quantity(field, title, formatter = null, sorter = null) {
    if (!formatter) {
      formatter = (cell) => {
        const items = cell.getValue();
        return items?.length || 0;
      }
      sorter = (a, b) => a.length - b.length;
    }

    const config = {
      field, title,
      hozAlign: 'right',
      formatter, sorter
    }
    return { ...TableColumns._defaultTextConfig(), ...config };
  }

  // ICONS
  static flag(action, text = '') {
    return this.icon(Template.flagIcon(), action, text)
  }

  static play(action, text = '') {
    return this.icon(Template.playIcon(), action, text)
  }

  static edit(action, text = '') {
    return this.icon(Template.editIcon(), action, text)
  }

  static trash(action, text = '') {
    return this.icon(Template.trashIcon(), action, text)
  }

  static icon(htmlIcon, action, text = '') {
    return this._iconConfig(htmlIcon, action, text)
  }

  static defaultBoolean(field, title) {
    return this.bool(field, title);
  }

  static bool(field, title, trueText = 'Yes', falseText = 'No') {
    const boolConfig = {
      field, title,
      formatter: (cell) => {
        const value = cell.getValue();

        if (value === true) {
          return `<div class="badge true">${trueText}</div>`;
        } else if (value === false) {
          return `<div class="badge false">${falseText}</div>`;
        }
      },
      tooltip: false
    };
    return { ...TableColumns._defaultConfig(), ...boolConfig };
  }

  // CUSTOM
  static hidden(field) {
    const hiddenConfig = { field, visible: false };
    return { ...TableColumns._defaultConfig(), ...hiddenConfig };
  }

  static _iconConfig(icon, action, title = '') {
    const iconConfig = {
      title, headerSort: false,
      formatter: () => icon,
      cellClick: async (event, cell) => await action(cell.getRow().getData(), event)
    }
    return { ...TableColumns._defaultConfig(), ...iconConfig };
  }

  static _defaultTextConfig() {
    const textConfig = { headerFilter: true };
    return { ...TableColumns._defaultConfig(), ...textConfig };
  }

  static _defaultConfig() {
    return {
      tooltip: true,
      resizable: false
    }
  }

}
